@@ -18,9 +18,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ *
+ *
  */
 
 /**
@@ -34,12 +34,6 @@
  * clutter_texture_set_from_file() functions are used to copy image
  * data into texture memory and subsequently realize the texture.
  *
- * If texture reads are supported by underlying GL implementation,
- * unrealizing frees image data from texture memory moving to main
- * system memory. Re-realizing then performs the opposite operation.
- * This process allows basic management of commonly limited available
- * texture memory.
- *
  * Note: a ClutterTexture will scale its contents to fit the bounding
  * box requested using clutter_actor_set_size(). To display an area of
  * a texture without scaling, you should set the clip area using
@@ -86,16 +80,9 @@ struct _ClutterTexturePrivate
   ClutterActor                *fbo_source;
   CoglHandle                   fbo_handle;
 
-  /* Non video memory copy of image data */
-  guint                        local_data_width, local_data_height;
-  guint                        local_data_rowstride;
-  guint                        local_data_has_alpha;
-  guchar                      *local_data;
-
   guint                        sync_actor_size : 1;
   guint                        repeat_x : 1;
   guint                        repeat_y : 1;
-  guint                        in_dispose : 1;
   guint                        keep_aspect_ratio : 1;
   guint                        load_size_async : 1;
   guint                        load_data_async : 1;
@@ -172,12 +159,6 @@ static GStaticMutex upload_list_mutex = G_STATIC_MUTEX_INIT;
 static void
 texture_fbo_free_resources (ClutterTexture *texture);
 
-static void
-clutter_texture_save_to_local_data (ClutterTexture *texture);
-
-static void
-clutter_texture_load_from_local_data (ClutterTexture *texture);
-
 GQuark
 clutter_texture_error_quark (void)
 {
@@ -241,17 +222,9 @@ clutter_texture_unrealize (ClutterActor *actor)
   if (priv->material == COGL_INVALID_HANDLE)
     return;
 
-  /* there's no need to read the pixels back when unrealizing inside
-   * a dispose run, and the dispose() call will release the GL
-   * texture data as well, so we can safely bail out now
-   */
-  if ((CLUTTER_PRIVATE_FLAGS (actor) & CLUTTER_ACTOR_IN_DESTRUCTION) ||
-      priv->in_dispose)
-    return;
-
   CLUTTER_MARK();
 
-  if (priv->fbo_source != COGL_INVALID_HANDLE)
+  if (priv->fbo_source != NULL)
     {
       /* Free up our fbo handle and texture resources, realize will recreate */
       cogl_handle_unref (priv->fbo_handle);
@@ -260,24 +233,6 @@ clutter_texture_unrealize (ClutterActor *actor)
       return;
     }
 
-  if (clutter_feature_available (CLUTTER_FEATURE_TEXTURE_READ_PIXELS))
-    {
-      /* Move image data from video to main memory.
-       * GL/ES cant do this - it probably makes sense
-       * to move this kind of thing into a ClutterProxyTexture
-       * where this behaviour can be better controlled.
-       *
-       * Or make it controllable via a property.
-       */
-      if (priv->local_data == NULL)
-	{
-	  clutter_texture_save_to_local_data (texture);
-	  CLUTTER_NOTE (TEXTURE, "moved pixels into system mem");
-	}
-
-      texture_free_gl_resources (texture);
-    }
-
   CLUTTER_NOTE (TEXTURE, "Texture unrealized");
 }
 
@@ -327,25 +282,14 @@ clutter_texture_realize (ClutterActor *actor)
       return;
     }
 
-  if (priv->local_data != NULL)
-    {
-      /* Move any local image data we have from unrealization
-       * back into video memory.
-      */
-      clutter_texture_load_from_local_data (texture);
-    }
-  else
-    {
-      /* If we have no data, then realization is a no-op but
-       * we still want to be in REALIZED state to maintain
-       * invariants. We may have already created the texture
-       * if someone set some data earlier, or we may create it
-       * later if someone sets some data later. The fact that
-       * we may have created it earlier is really a bug, since
-       * it means ClutterTexture can have GL resources without
-       * being realized.
-       */
-    }
+  /* If the texture is not a FBO, then realization is a no-op but
+   * we still want to be in REALIZED state to maintain invariants.
+   * We may have already created the texture if someone set some
+   * data earlier, or we may create it later if someone sets some
+   * data later. The fact that we may have created it earlier is
+   * really a bug, since it means ClutterTexture can have GL
+   * resources without being realized.
+   */
 
   CLUTTER_NOTE (TEXTURE, "Texture realized");
 }
@@ -449,66 +393,127 @@ clutter_texture_allocate (ClutterActor           *self,
 }
 
 static void
-clutter_texture_set_fbo_projection (ClutterActor *self)
+set_viewport_with_buffer_under_fbo_source (ClutterActor *fbo_source,
+                                           int viewport_width,
+                                           int viewport_height)
 {
-  ClutterTexturePrivate *priv = CLUTTER_TEXTURE (self)->priv;
   ClutterVertex verts[4];
-  gfloat viewport[4];
-  gfloat x_min, x_max, y_min, y_max;
-  gfloat tx_min, tx_max, ty_min, ty_max;
-  gfloat tan_angle, near_size;
-  ClutterPerspective perspective;
-  ClutterStage *stage;
+  float x_min = G_MAXFLOAT, y_min = G_MAXFLOAT;
+  int x_offset, y_offset;
   int i;
 
-  /* Get the bounding rectangle of the source as drawn in screen
-     coordinates */
-  clutter_actor_get_abs_allocation_vertices (priv->fbo_source, verts);
+  /* Get the actors allocation transformed into screen coordinates.
+   *
+   * XXX: Note: this may not be a bounding box for the actor, since an
+   * actor with depth may escape the box due to its perspective
+   * projection. */
+  clutter_actor_get_abs_allocation_vertices (fbo_source, verts);
 
-  x_min = x_max = verts[0].x;
-  y_min = y_max = verts[0].y;
-
-  for (i = 1; i < G_N_ELEMENTS (verts); ++i)
+  for (i = 0; i < G_N_ELEMENTS (verts); ++i)
     {
       if (verts[i].x < x_min)
 	x_min = verts[i].x;
-
-      if (verts[i].x > x_max)
-	x_max = verts[i].x;
-
       if (verts[i].y < y_min)
 	y_min = verts[i].y;
-
-      if (verts[i].y > y_max)
-	y_max = verts[i].y;
     }
 
-  stage = CLUTTER_STAGE (clutter_actor_get_stage (self));
-  clutter_stage_get_perspective (stage, &perspective);
+  /* XXX: It's not good enough to round by simply truncating the fraction here
+   * via a cast, as it results in offscreen rendering being offset by 1 pixel
+   * in many cases... */
+#define ROUND(x) ((x) >= 0 ? (long)((x) + 0.5) : (long)((x) - 0.5))
 
-  /* Convert the coordinates back to [-1,1] range */
-  cogl_get_viewport (viewport);
+  x_offset = ROUND (-x_min);
+  y_offset = ROUND (-y_min);
 
-  tx_min = (x_min / viewport[2])
-         * 2 - 1.0;
-  tx_max = (x_max / viewport[2])
-         * 2 - 1.0;
-  ty_min = (y_min / viewport[3])
-         * 2 - 1.0;
-  ty_max = (y_max / viewport[3])
-         * 2 - 1.0;
+#undef ROUND
 
-  /* Set up a projection matrix so that the actor will be projected as
-     if it was drawn at its original location */
-  tan_angle = tanf ((perspective.fovy / 2) * (G_PI / 180.0));
-  near_size = perspective.z_near * tan_angle;
+  /* translate the viewport so that the source actor lands on the
+   * sub-region backed by the offscreen framebuffer... */
+  cogl_set_viewport (x_offset, y_offset, viewport_width, viewport_height);
+}
 
-  cogl_frustum ((tx_min * near_size),
-                (tx_max * near_size),
-                (-ty_min * near_size),
-                (-ty_max * near_size),
-                perspective.z_near,
-                perspective.z_far);
+static void
+update_fbo (ClutterActor *self)
+{
+  ClutterTexture        *texture = CLUTTER_TEXTURE (self);
+  ClutterTexturePrivate *priv = texture->priv;
+  ClutterMainContext    *context;
+  ClutterShader         *shader = NULL;
+  ClutterActor          *stage = NULL;
+  ClutterPerspective     perspective;
+  CoglColor              transparent_col;
+
+  context = _clutter_context_get_default ();
+
+  if (context->shaders)
+    shader = clutter_actor_get_shader (context->shaders->data);
+
+  /* Temporarily turn off the shader on the top of the context's shader stack,
+   * to restore the GL pipeline to it's natural state.
+   */
+  if (shader)
+    clutter_shader_set_is_enabled (shader, FALSE);
+
+  /* Redirect drawing to the fbo */
+  cogl_push_framebuffer (priv->fbo_handle);
+
+  if ((stage = clutter_actor_get_stage (self)))
+    {
+      gfloat stage_width, stage_height;
+      ClutterActor *source_parent;
+
+      /* We copy the projection and modelview matrices from the stage to
+       * the offscreen framebuffer and create a viewport larger than the
+       * offscreen framebuffer - the same size as the stage.
+       *
+       * The fbo source actor gets rendered into this stage size viewport at the
+       * same position it normally would after applying all it's usual parent
+       * transforms and it's own scale and rotate transforms etc.
+       *
+       * The viewport is offset such that the offscreen buffer will be positioned
+       * under the actor.
+       */
+
+      clutter_stage_get_perspective (CLUTTER_STAGE (stage), &perspective);
+      clutter_actor_get_size (stage, &stage_width, &stage_height);
+
+      /* Set the projection matrix modelview matrix and viewport size as
+       * they are for the stage... */
+      _cogl_setup_viewport (stage_width, stage_height,
+                            perspective.fovy,
+                            perspective.aspect,
+                            perspective.z_near,
+                            perspective.z_far);
+
+      /* Negatively offset the viewport so that the offscreen framebuffer is
+       * position underneath the fbo_source actor... */
+      set_viewport_with_buffer_under_fbo_source (priv->fbo_source,
+                                                 stage_width,
+                                                 stage_height);
+
+      /* Reapply the source's parent transformations */
+      if ((source_parent = clutter_actor_get_parent (priv->fbo_source)))
+        _clutter_actor_apply_modelview_transform_recursive (source_parent,
+                                                            NULL);
+    }
+
+
+  /* cogl_clear is called to clear the buffers */
+  cogl_color_set_from_4ub (&transparent_col, 0, 0, 0, 0);
+  cogl_clear (&transparent_col,
+              COGL_BUFFER_BIT_COLOR |
+              COGL_BUFFER_BIT_DEPTH);
+  cogl_disable_fog ();
+
+  /* Render the actor to the fbo */
+  clutter_actor_paint (priv->fbo_source);
+
+  /* Restore drawing to the previous framebuffer */
+  cogl_pop_framebuffer ();
+
+  /* If there is a shader on top of the shader stack, turn it back on. */
+  if (shader)
+    clutter_shader_set_is_enabled (shader, TRUE);
 }
 
 static void
@@ -517,7 +522,6 @@ clutter_texture_paint (ClutterActor *self)
   ClutterTexture *texture = CLUTTER_TEXTURE (self);
   ClutterTexturePrivate *priv = texture->priv;
   ClutterActorBox box = { 0, };
-  CoglColor       transparent_col;
   gfloat          t_w, t_h;
   guint8          paint_opacity = clutter_actor_get_paint_opacity (self);
 
@@ -530,85 +534,7 @@ clutter_texture_paint (ClutterActor *self)
     }
 
   if (priv->fbo_handle != COGL_INVALID_HANDLE)
-    {
-      ClutterMainContext *context;
-      ClutterShader      *shader = NULL;
-      ClutterActor       *stage = NULL;
-      ClutterPerspective  perspective;
-
-      context = _clutter_context_get_default ();
-
-      if (context->shaders)
-        shader = clutter_actor_get_shader (context->shaders->data);
-
-      /* Temporarily turn of the shader on the top of the context's
-       * shader stack, to restore the GL pipeline to it's natural state.
-       */
-      if (shader)
-        clutter_shader_set_is_enabled (shader, FALSE);
-
-      /* Redirect drawing to the fbo */
-      cogl_set_draw_buffer (COGL_OFFSCREEN_BUFFER, priv->fbo_handle);
-
-      if ((stage = clutter_actor_get_stage (self)))
-	{
-	  gfloat stage_width, stage_height;
-	  ClutterActor *source_parent;
-
-	  clutter_stage_get_perspective (CLUTTER_STAGE (stage), &perspective);
-	  clutter_actor_get_size (stage, &stage_width, &stage_height);
-
-	  /* Use below to set the modelview matrix as if the viewport
-	     was still the same size as the stage */
-          _cogl_setup_viewport (stage_width, stage_height,
-                                perspective.fovy,
-                                perspective.aspect,
-                                perspective.z_near,
-                                perspective.z_far);
-
-	  /* Use a projection matrix that makes the actor appear as it
-	     would if it was rendered at its normal screen location */
-	  clutter_texture_set_fbo_projection (self);
-
-	  /* Reset the viewport to the size of the FBO */
-	  cogl_viewport (priv->image_width, priv->image_height);
-
-	  /* Reapply the source's parent transformations */
-	  if ((source_parent = clutter_actor_get_parent (priv->fbo_source)))
-	    _clutter_actor_apply_modelview_transform_recursive (source_parent,
-								NULL);
-	}
-
-      /* cogl_clear is called to clear the buffers */
-      cogl_color_set_from_4ub (&transparent_col, 0, 0, 0, 0);
-      cogl_clear (&transparent_col,
-		  COGL_BUFFER_BIT_COLOR |
-		  COGL_BUFFER_BIT_DEPTH);
-      cogl_disable_fog ();
-
-      /* Clear the clipping stack so that if the FBO actor is being
-	 clipped then it won't affect drawing the source */
-      cogl_clip_stack_save ();
-
-      /* Render out actor scene to fbo */
-      clutter_actor_paint (priv->fbo_source);
-
-      cogl_clip_stack_restore ();
-
-      /* Restore drawing to the frame buffer */
-      cogl_set_draw_buffer (COGL_WINDOW_BUFFER, COGL_INVALID_HANDLE);
-
-      /* Restore the perspective matrix using cogl_perspective so that
-	 the inverse matrix will be right */
-      cogl_perspective (perspective.fovy,
-                        perspective.aspect,
-                        perspective.z_near,
-                        perspective.z_far);
-
-      /* If there is a shader on top of the shader stack, turn it back on. */
-      if (shader)
-        clutter_shader_set_is_enabled (shader, TRUE);
-    }
+    update_fbo (self);
 
   CLUTTER_NOTE (PAINT,
                 "painting texture '%s'",
@@ -719,27 +645,10 @@ static void
 clutter_texture_dispose (GObject *object)
 {
   ClutterTexture *texture = CLUTTER_TEXTURE (object);
-  ClutterTexturePrivate *priv;
-
-  priv = texture->priv;
-
-  /* mark that we are in dispose, so when the parent class'
-   * dispose implementation will call unrealize on us we'll
-   * not try to copy back the resources from video memory
-   * to system memory
-   */
-  if (!priv->in_dispose)
-    priv->in_dispose = TRUE;
 
   texture_free_gl_resources (texture);
   texture_fbo_free_resources (texture);
 
-  if (priv->local_data != NULL)
-    {
-      g_free (priv->local_data);
-      priv->local_data = NULL;
-    }
-
   clutter_texture_async_load_cancel (texture);
 
   G_OBJECT_CLASS (clutter_texture_parent_class)->dispose (object);
@@ -1195,78 +1104,9 @@ clutter_texture_init (ClutterTexture *self)
   priv->sync_actor_size   = TRUE;
   priv->material          = cogl_material_new ();
   priv->fbo_handle        = COGL_INVALID_HANDLE;
-  priv->local_data        = NULL;
   priv->keep_aspect_ratio = FALSE;
 }
 
-static void
-clutter_texture_save_to_local_data (ClutterTexture *texture)
-{
-  ClutterTexturePrivate *priv;
-  int                    bpp;
-  CoglPixelFormat        pixel_format;
-  CoglHandle             cogl_texture;
-
-  priv = texture->priv;
-
-  if (priv->local_data)
-    {
-      g_free (priv->local_data);
-      priv->local_data = NULL;
-    }
-
-  if (priv->material == COGL_INVALID_HANDLE)
-    return;
-
-  cogl_texture = clutter_texture_get_cogl_texture (texture);
-
-  priv->local_data_width = cogl_texture_get_width (cogl_texture);
-  priv->local_data_height = cogl_texture_get_height (cogl_texture);
-  pixel_format = cogl_texture_get_format (cogl_texture);
-  priv->local_data_has_alpha = pixel_format & COGL_A_BIT;
-  bpp = priv->local_data_has_alpha ? 4 : 3;
-
-  /* Align to 4 bytes */
-  priv->local_data_rowstride = (priv->local_data_width * bpp + 3) & ~3;
-
-  priv->local_data = g_malloc (priv->local_data_rowstride
-			       * priv->local_data_height);
-
-  if (cogl_texture_get_data (cogl_texture,
-			     priv->local_data_has_alpha
-			     ? COGL_PIXEL_FORMAT_RGBA_8888_PRE
-			     : COGL_PIXEL_FORMAT_RGB_888,
-			     priv->local_data_rowstride,
-			     priv->local_data) == 0)
-    {
-      g_free (priv->local_data);
-      priv->local_data = NULL;
-    }
-}
-
-static void
-clutter_texture_load_from_local_data (ClutterTexture *texture)
-{
-  ClutterTexturePrivate *priv;
-
-  priv = texture->priv;
-
-  if (priv->local_data == NULL)
-    return;
-
-  clutter_texture_set_from_rgb_data (texture,
-				     priv->local_data,
-				     priv->local_data_has_alpha,
-				     priv->local_data_width,
-				     priv->local_data_height,
-				     priv->local_data_rowstride,
-				     priv->local_data_has_alpha ? 4: 3,
-				     CLUTTER_TEXTURE_RGB_FLAG_PREMULT, NULL);
-
-  g_free (priv->local_data);
-  priv->local_data = NULL;
-}
-
 /**
  * clutter_texture_get_cogl_material:
  * @texture: A #ClutterTexture
@@ -1292,9 +1132,14 @@ clutter_texture_get_cogl_material (ClutterTexture *texture)
  * @texture: A #ClutterTexture
  * @cogl_material: A CoglHandle for a material
  *
- * Replaces the underlying COGL texture drawn by this actor with
- * @cogl_tex. A reference to the texture is taken so if the handle is
- * no longer needed it should be deref'd with cogl_handle_unref.
+ * Replaces the underlying Cogl material drawn by this actor with
+ * @cogl_material. A reference to the material is taken so if the
+ * handle is no longer needed it should be deref'd with
+ * cogl_handle_unref. Texture data is attached to the material so
+ * calling this function also replaces the Cogl
+ * texture. #ClutterTexture requires that the material have a texture
+ * layer so you should set one on the material before calling this
+ * function.
  *
  * Since: 0.8
  *
@@ -1307,6 +1152,8 @@ clutter_texture_set_cogl_material (ClutterTexture *texture,
 
   g_return_if_fail (CLUTTER_IS_TEXTURE (texture));
 
+  cogl_handle_ref (cogl_material);
+
   /* This */
   if (texture->priv->material)
     cogl_handle_unref (texture->priv->material);
@@ -1404,15 +1251,7 @@ clutter_texture_set_cogl_texture (ClutterTexture  *texture,
   /* Remove old texture */
   texture_free_gl_resources (texture);
 
-  /* Free any saved data so realization doesn't resend it to GL */
-  if (priv->local_data)
-    {
-      g_free (priv->local_data);
-      priv->local_data = NULL;
-    }
-
   /* Use the new texture */
-
   cogl_material_set_layer (priv->material, 0, cogl_tex);
 
   /* The material now holds a reference to the texture so we can
@@ -1443,8 +1282,7 @@ clutter_texture_set_cogl_texture (ClutterTexture  *texture,
   g_object_notify (G_OBJECT (texture), "cogl-texture");
 
   /* If resized actor may need resizing but paint() will do this */
-  if (CLUTTER_ACTOR_IS_VISIBLE (texture))
-    clutter_actor_queue_redraw (CLUTTER_ACTOR (texture));
+  clutter_actor_queue_redraw (CLUTTER_ACTOR (texture));
 }
 
 static gboolean
@@ -1467,7 +1305,6 @@ clutter_texture_set_from_data (ClutterTexture     *texture,
   /* FIXME if we are not realized, we should store the data
    * for future use, instead of creating the texture.
    */
-
   new_texture = cogl_texture_new_from_data (width, height,
                                             flags,
                                             source_format,
@@ -1477,11 +1314,18 @@ clutter_texture_set_from_data (ClutterTexture     *texture,
 
   if (G_UNLIKELY (new_texture == COGL_INVALID_HANDLE))
     {
-      g_set_error (error, CLUTTER_TEXTURE_ERROR,
+      GError *inner_error = NULL;
+
+      g_set_error (&inner_error, CLUTTER_TEXTURE_ERROR,
                    CLUTTER_TEXTURE_ERROR_BAD_FORMAT,
                    "Failed to create COGL texture");
 
-      g_signal_emit (texture, texture_signals[LOAD_FINISHED], 0, *error);
+      g_signal_emit (texture, texture_signals[LOAD_FINISHED], 0, inner_error);
+
+      if (error != NULL)
+        g_propagate_error (error, inner_error);
+      else
+        g_error_free (inner_error);
 
       return FALSE;
     }
@@ -1525,13 +1369,10 @@ clutter_texture_set_from_rgb_data (ClutterTexture       *texture,
 				   ClutterTextureFlags   flags,
 				   GError              **error)
 {
-  ClutterTexturePrivate *priv;
   CoglPixelFormat source_format;
 
   g_return_val_if_fail (CLUTTER_IS_TEXTURE (texture), FALSE);
 
-  priv = texture->priv;
-
   /* Convert the flags to a CoglPixelFormat */
   if (has_alpha)
     {
@@ -1600,8 +1441,6 @@ clutter_texture_set_from_yuv_data (ClutterTexture     *texture,
 				   ClutterTextureFlags flags,
 				   GError            **error)
 {
-  ClutterTexturePrivate *priv;
-
   g_return_val_if_fail (CLUTTER_IS_TEXTURE (texture), FALSE);
 
   if (!clutter_feature_available (CLUTTER_FEATURE_TEXTURE_YUV))
@@ -1612,8 +1451,6 @@ clutter_texture_set_from_yuv_data (ClutterTexture     *texture,
       return FALSE;
     }
 
-  priv = texture->priv;
-
   /* Convert the flags to a CoglPixelFormat */
   if ((flags & CLUTTER_TEXTURE_YUV_FLAG_YUV2))
     {
@@ -2034,8 +1871,7 @@ clutter_texture_set_filter_quality (ClutterTexture        *texture,
 
       g_object_notify (G_OBJECT (texture), "filter-quality");
 
-      if (CLUTTER_ACTOR_IS_VISIBLE (texture))
-	clutter_actor_queue_redraw (CLUTTER_ACTOR (texture));
+      clutter_actor_queue_redraw (CLUTTER_ACTOR (texture));
     }
 }
 
@@ -2208,11 +2044,8 @@ clutter_texture_set_area_from_rgb_data (ClutterTexture     *texture,
                                         ClutterTextureFlags flags,
                                         GError            **error)
 {
-  ClutterTexturePrivate *priv;
-  CoglPixelFormat        source_format;
-  CoglHandle             cogl_texture;
-
-  priv = texture->priv;
+  CoglPixelFormat source_format;
+  CoglHandle cogl_texture;
 
   if (has_alpha)
     {
@@ -2223,6 +2056,7 @@ clutter_texture_set_area_from_rgb_data (ClutterTexture     *texture,
 		       "Unsupported BPP");
 	  return FALSE;
 	}
+
       source_format = COGL_PIXEL_FORMAT_RGBA_8888;
     }
   else
@@ -2234,17 +2068,22 @@ clutter_texture_set_area_from_rgb_data (ClutterTexture     *texture,
 		       "Unsupported BPP");
 	  return FALSE;
 	}
+
       source_format = COGL_PIXEL_FORMAT_RGB_888;
     }
+
   if ((flags & CLUTTER_TEXTURE_RGB_FLAG_BGR))
     source_format |= COGL_BGR_BIT;
+
   if ((flags & CLUTTER_TEXTURE_RGB_FLAG_PREMULT))
     source_format |= COGL_PREMULT_BIT;
 
   /* attempt to realize ... */
   if (!CLUTTER_ACTOR_IS_REALIZED (texture) &&
       clutter_actor_get_stage (CLUTTER_ACTOR (texture)) != NULL)
-    clutter_actor_realize (CLUTTER_ACTOR (texture));
+    {
+      clutter_actor_realize (CLUTTER_ACTOR (texture));
+    }
 
   /* due to the fudging of clutter_texture_set_cogl_texture()
    * which allows setting a texture pre-realize, we may end
@@ -2276,8 +2115,7 @@ clutter_texture_set_area_from_rgb_data (ClutterTexture     *texture,
   /* rename signal */
   g_signal_emit (texture, texture_signals[PIXBUF_CHANGE], 0);
 
-  if (CLUTTER_ACTOR_IS_VISIBLE (texture))
-    clutter_actor_queue_redraw (CLUTTER_ACTOR (texture));
+  clutter_actor_queue_redraw (CLUTTER_ACTOR (texture));
 
   return TRUE;
 }
